@@ -35,7 +35,8 @@ class GroupCatchExceptions(click.Group):
             sys.exit(1)
 
         except apiclient.ResourceCannotBeDeletedException as e:
-            click.echo('ERROR: Cannot delete resource: %s' % error_text(e.text))
+            click.echo('ERROR: Cannot delete resource: %s' %
+                       error_text(e.text))
             sys.exit(1)
 
         except apiclient.ResourceNotFoundException as e:
@@ -43,16 +44,16 @@ class GroupCatchExceptions(click.Group):
             sys.exit(1)
 
         except apiclient.ResourceInUseException as e:
-            click.echo('ERROR: Resource in use:\n%s' % error_text(e.text))
+            click.echo('ERROR: Resource in use: %s' % error_text(e.text))
             sys.exit(1)
 
         except apiclient.InternalServerError as e:
             # Print full error since server should not fail
-            click.echo('ERROR: Internal Server Error:\n%s' % e.text)
+            click.echo('ERROR: Internal Server Error: %s' % e.text)
             sys.exit(1)
 
         except apiclient.InsufficientResourcesException as e:
-            click.echo('ERROR: Insufficient Resources:\n%s' %
+            click.echo('ERROR: Insufficient Resources: %s' %
                        error_text(e.text))
             sys.exit(1)
 
@@ -65,7 +66,7 @@ def error_text(json_text):
         err = json.loads(json_text)
         if 'error' in err:
             return err['error']
-    except:
+    except Exception:
         pass
 
     return json_text
@@ -144,7 +145,7 @@ def node_list(ctx):
         print('name,ip,lastseen,version')
         for n in nodes:
             print('%s,%s,%s,%s' % (
-                    n['name'], n['ip'], n['lastseen'], n['version']))
+                n['name'], n['ip'], n['lastseen'], n['version']))
 
     elif ctx.obj['OUTPUT'] == 'json':
         filtered_nodes = []
@@ -378,13 +379,14 @@ def _show_network(ctx, n):
     metadata = CLIENT.get_network_metadata(n['uuid'])
 
     if ctx.obj['OUTPUT'] == 'json':
-        filtered = filter_dict(n, ['uuid', 'name', 'vxid', 'netblock', 'provide_dhcp',
-                                   'provide_nat', 'namespace'])
+        filtered = filter_dict(n, ['uuid', 'name', 'vxid', 'netblock',
+                                   'provide_dhcp', 'provide_nat',
+                                   'floating_gateway', 'namespace'])
         filtered['metadata'] = metadata
         print(json.dumps(filtered, indent=4, sort_keys=True))
         return
 
-    format_string = '%-12s: %s'
+    format_string = '%-16s: %s'
     if ctx.obj['OUTPUT'] == 'simple':
         format_string = '%s:%s'
 
@@ -394,6 +396,7 @@ def _show_network(ctx, n):
     print(format_string % ('netblock', n['netblock']))
     print(format_string % ('provide dhcp', n['provide_dhcp']))
     print(format_string % ('provide nat', n['provide_nat']))
+    print(format_string % ('floating gateway', n['floating_gateway']))
     print(format_string % ('namespace', n['namespace']))
     print(format_string % ('state', n['state']))
 
@@ -492,6 +495,35 @@ def network_delete(ctx, network_uuid=None):
         print('{}')
 
 
+@network.command(name='instances', help='List instances on a network')
+@click.argument('network_uuid',
+                type=click.STRING, autocompletion=_get_networks)
+@click.pass_context
+def network_list_instances(ctx, network_uuid=None):
+    interfaces = CLIENT.get_network_interfaces(network_uuid)
+
+    if ctx.obj['OUTPUT'] == 'pretty':
+        x = PrettyTable()
+        x.field_names = ['instance_uuid', 'ipv4', 'floating']
+        for ni in interfaces:
+            x.add_row([ni['instance_uuid'], ni['ipv4'], ni['floating']])
+        print(x)
+
+    elif ctx.obj['OUTPUT'] == 'simple':
+        print('instance_uuid,ipv4,floating')
+        for ni in interfaces:
+            print('%s,%s,%s' %
+                  (ni['instance_uuid'], ni['ipv4'], ni['floating']))
+
+    elif ctx.obj['OUTPUT'] == 'json':
+        filtered_ni = []
+        for ni in interfaces:
+            filtered_ni.append(filter_dict(
+                ni, ['instance_uuid', 'ipv4', 'floating']))
+        print(json.dumps({'instances': filtered_ni},
+                         indent=4, sort_keys=True))
+
+
 @network.command(name='set-metadata', help='Set a metadata item')
 @click.argument('network_uuid', type=click.STRING, autocompletion=_get_networks)
 @click.argument('key', type=click.STRING)
@@ -562,6 +594,24 @@ def instance_list(ctx, all=False):
                          indent=4, sort_keys=True))
 
 
+def _pretty_data(row, space_rules):
+    ret = ''
+    for key in space_rules:
+        ret += key + '=' + str(row.get(key, '')).ljust(space_rules[key]) + '  '
+    return ret
+
+
+def _pretty_dict(lead_space, rows, space_rules):
+    ret = ''
+
+    if rows:
+        ret += _pretty_data(rows[0], space_rules)
+    for r in rows[1:]:
+        ret += '\n'.ljust(lead_space) + _pretty_data(r, space_rules)
+
+    return ret
+
+
 def _show_instance(ctx, i, include_snapshots=False):
     if not i:
         print('Instance not found')
@@ -574,8 +624,9 @@ def _show_instance(ctx, i, include_snapshots=False):
 
     if ctx.obj['OUTPUT'] == 'json':
         out = filter_dict(i, ['uuid', 'name', 'namespace', 'cpus', 'memory',
-                              'disk_spec', 'node', 'console_port', 'vdi_port',
-                              'ssh_key', 'user_data', 'power_state', 'state'])
+                              'disk_spec', 'video', 'node', 'console_port',
+                              'vdi_port', 'ssh_key', 'user_data',
+                              'power_state', 'state'])
         out['network_interfaces'] = []
         for interface in interfaces:
             _show_interface(ctx, interface, out)
@@ -591,16 +642,24 @@ def _show_instance(ctx, i, include_snapshots=False):
         print(json.dumps(out, indent=4, sort_keys=True))
         return
 
-    format_string = '%-12s: %s'
     if ctx.obj['OUTPUT'] == 'simple':
         format_string = '%s:%s'
+    else:
+        format_string = '%-12s: %s'
+        d_space = {'type': 5, 'bus': 4, 'size': 2, 'base': 0}
+        v_space = {'model': 0, 'memory': 0}
 
     print(format_string % ('uuid', i['uuid']))
     print(format_string % ('name', i['name']))
     print(format_string % ('namespace', i['namespace']))
     print(format_string % ('cpus', i['cpus']))
     print(format_string % ('memory', i['memory']))
-    print(format_string % ('disk spec', i['disk_spec']))
+    if ctx.obj['OUTPUT'] == 'pretty':
+        print(format_string % ('disk spec',
+                               _pretty_dict(15, i['disk_spec'], d_space)))
+    if ctx.obj['OUTPUT'] == 'pretty':
+        print(format_string % ('video',
+                               _pretty_dict(15, (i['video'],), v_space)))
     print(format_string % ('node', i['node']))
     print(format_string % ('power state', i['power_state']))
     print(format_string % ('state', i['state']))
@@ -614,6 +673,18 @@ def _show_instance(ctx, i, include_snapshots=False):
     print(format_string % ('ssh key', i['ssh_key']))
     print(format_string % ('user data', i['user_data']))
 
+    if ctx.obj['OUTPUT'] == 'simple':
+        print()
+        print('disk_spec,type,bus,size,base')
+        for d in i['disk_spec']:
+            print('disk_spec,%s,%s,%s,%s' % (
+                d['type'], d['bus'], d['size'], d['base']))
+
+    if ctx.obj['OUTPUT'] == 'simple':
+        print()
+        print('video,model,memory')
+        print('video,%s,%s' % (i['video']['model'], i['video']['memory']))
+
     print()
     if ctx.obj['OUTPUT'] == 'pretty':
         format_string = '    %-8s: %s'
@@ -671,6 +742,9 @@ def _parse_spec(spec):
     return spec.split('@')
 
 
+# TODO(mikal): this misses the detailed version of disk and network specs, as well
+# as guidance on how to use the video command line. We need to rethink how we're
+# doing this, as its getting pretty long.
 @instance.command(name='create',
                   help=('Create an instance.\n\n'
                         'NAME:      The name of the instance.\n'
@@ -682,6 +756,7 @@ def _parse_spec(spec):
                         '--disk/-d:      The disks attached to the instance, in this format: \n'
                         '                size@image_url where size is in GB and @image_url\n'
                         '                is optional.\n'
+                        '--video/-V:     The video configuration for the instance.\n'
                         '--sshkey/-i:    The path to a ssh public key to configure on the\n'
                         '                instance via config drive / cloud-init.\n'
                         '--sshkeydata/-I:\n'
@@ -709,11 +784,12 @@ def _parse_spec(spec):
 @click.option('-u', '--userdata', type=click.STRING)
 @click.option('-U', '--encodeduserdata', type=click.STRING)
 @click.option('-p', '--placement', type=click.STRING)
+@click.option('-V', '--videospec', type=click.STRING)
 @click.option('--namespace', type=click.STRING)
 @click.pass_context
 def instance_create(ctx, name=None, cpus=None, memory=None, network=None, networkspec=None,
                     disk=None, diskspec=None, sshkey=None, sshkeydata=None, userdata=None,
-                    encodeduserdata=None, placement=None, namespace=None):
+                    encodeduserdata=None, placement=None, videospec=None, namespace=None):
     if len(disk) < 1 and len(diskspec) < 1:
         print('You must specify at least one disk')
         return
@@ -746,7 +822,7 @@ def instance_create(ctx, name=None, cpus=None, memory=None, network=None, networ
         size, base = p
         try:
             size_int = int(size)
-        except:
+        except Exception:
             print('Disk size is not an integer')
             return
 
@@ -787,11 +863,21 @@ def instance_create(ctx, name=None, cpus=None, memory=None, network=None, networ
             defn[s[0]] = s[1]
         netdefs.append(defn)
 
+    video = {'model': 'cirrus', 'memory': 16384}
+    if videospec:
+        for elem in videospec.split(','):
+            s = elem.split('=')
+            if len(s) != 2:
+                print('Error in video specification - '
+                      ' should be key=value: %s' % elem)
+                return
+            video[s[0]] = s[1]
+
     _show_instance(
         ctx,
         CLIENT.create_instance(name, cpus, memory, netdefs, diskdefs, sshkey_content,
                                userdata_content, force_placement=placement,
-                               namespace=namespace))
+                               namespace=namespace, video=video))
 
 
 @instance.command(name='delete', help='Delete an instance')
@@ -963,7 +1049,7 @@ def _show_interface(ctx, interface, out=[]):
         out['network_interfaces'].append(
             filter_dict(
                 interface, ['uuid', 'network_uuid', 'macaddr', 'order',
-                            'ipv4', 'floating']))
+                            'ipv4', 'floating', 'model']))
         return
 
     if ctx.obj['OUTPUT'] == 'pretty':
@@ -974,11 +1060,12 @@ def _show_interface(ctx, interface, out=[]):
         print(format_string % ('order', interface['order']))
         print(format_string % ('ipv4', interface['ipv4']))
         print(format_string % ('floating', interface['floating']))
+        print(format_string % ('model', interface['model']))
     else:
-        print('iface,%s,%s,%s,%s,%s,%s'
+        print('iface,%s,%s,%s,%s,%s,%s,%s'
               % (interface['uuid'], interface['network_uuid'],
                  interface['macaddr'], interface['order'], interface['ipv4'],
-                 interface['floating']))
+                 interface['floating'], interface['model']))
 
 
 @interface.command(name='show', help='Show an interface')
@@ -997,7 +1084,8 @@ def interface_show(ctx, interface_uuid=None):
     if ctx.obj['OUTPUT'] == 'pretty':
         print('Interface:')
     else:
-        print('iface,interface uuid,network uuid,macaddr,order,ipv4,floating')
+        print('iface,interface uuid,network uuid,'
+              'macaddr,order,ipv4,floating,model')
 
     _show_interface(ctx, interface)
 
