@@ -8,7 +8,6 @@
 import base64
 import bcrypt
 import flask
-from flask import request
 from flask_jwt_extended import create_access_token
 from flask_jwt_extended import get_jwt_identity
 from flask_jwt_extended import JWTManager
@@ -37,7 +36,6 @@ from oslo_concurrency import processutils
 
 from shakenfist import config
 from shakenfist import db
-from shakenfist import db
 from shakenfist import images
 from shakenfist import net
 from shakenfist import scheduler
@@ -105,8 +103,16 @@ def generic_wrapper(func):
             for header in flask.request.headers:
                 formatted_headers.append(str(header))
 
-            LOG.info('API request: %s %s\n    Headers:\n        %s\n    Args: %s\n    KWargs: %s'
-                     % (flask.request.method, flask.request.url, '\n        '.join(formatted_headers), args, kwargs))
+            msg = 'API request: %s %s\n    Headers:\n        %s' % (
+                flask.request.method, flask.request.url,
+                '\n        '.join(formatted_headers))
+            msg += '\n    Args: %s\n    KWargs: %s' % (args, kwargs)
+
+            if re.match(r'http(|s)://0.0.0.0:\d+/$', flask.request.url):
+                LOG.debug(msg)
+            else:
+                LOG.info(msg)
+
             return func(*args, **kwargs)
 
         except TypeError as e:
@@ -232,7 +238,7 @@ def arg_is_network_uuid(func):
             kwargs['network_from_db'] = db.get_network(
                 kwargs['network_uuid'])
         if not kwargs.get('network_from_db'):
-            LOG.info('network(%s): network not found, genuinely missing' %
+            LOG.info('network(%s): network not found or deleted' %
                      kwargs['network_uuid'])
             return error(404, 'network not found')
 
@@ -551,12 +557,13 @@ def _delete_instance(instance_uuid, instance_from_db_virt):
     with db.get_lock('/sf/instance/%s' % instance_uuid) as _:
         db.add_event('instance', instance_uuid, 'api', 'delete', None, None)
 
+        # Create list of networks used by instance
         instance_networks = []
         for iface in list(db.get_instance_interfaces(instance_uuid)):
             if not iface['network_uuid'] in instance_networks:
                 instance_networks.append(iface['network_uuid'])
-                db.update_network_interface_state(iface['uuid'], 'deleted')
 
+        # Create list of networks used by all other instances
         host_networks = []
         for inst in list(
                 db.get_instances(only_node=config.parsed.get('NODE_NAME'))):
@@ -567,14 +574,17 @@ def _delete_instance(instance_uuid, instance_from_db_virt):
 
         instance_from_db_virt.delete()
 
+        # Check each network used by the deleted instance
         for network in instance_networks:
             n = net.from_db(network)
             if n:
+                # If network used by another instance, only update
                 if network in host_networks:
                     with util.RecordedOperation('deallocate ip address',
                                                 instance_from_db_virt) as _:
                         n.update_dhcp()
                 else:
+                    # Network not used by any other instance therefore delete
                     with util.RecordedOperation('remove network', n) as _:
                         n.delete()
 
@@ -610,7 +620,7 @@ class Instances(Resource):
     @jwt_required
     def post(self, name=None, cpus=None, memory=None, network=None,
              disk=None, ssh_key=None, user_data=None, placed_on=None, namespace=None,
-             instance_uuid=None):
+             instance_uuid=None, video=None):
         global SCHEDULER
 
         # Check that the instance name is safe for use as a DNS host name
@@ -633,6 +643,9 @@ class Instances(Resource):
                 if 'network_uuid' not in n:
                     return error(400, 'network specification is missing network_uuid')
 
+        if not video:
+            video = {'model': 'cirrus', 'memory': 16384}
+
         if not namespace:
             namespace = get_jwt_identity()
 
@@ -664,7 +677,8 @@ class Instances(Resource):
                 vcpus=cpus,
                 ssh_key=ssh_key,
                 user_data=user_data,
-                owner=namespace
+                owner=namespace,
+                video=video
             )
 
         if not SCHEDULER:
@@ -711,7 +725,7 @@ class Instances(Resource):
             if not placement == config.parsed.get('NODE_NAME'):
                 resp = self._instance_start_remote(
                     placement, instance_uuid, namespace)
-                if placed_on or resp.status_code == 200:
+                if placed_on or resp.status_code != 507:
                     return resp
                 placement = None
             else:
@@ -729,9 +743,12 @@ class Instances(Resource):
         body['instance_uuid'] = instance_uuid
         body['namespace'] = namespace
 
+        # NOTE(mikal): the user of the system namespace is deliberate here. The
+        # namespace to create the instance in is specified above, but we need to
+        # make this call as system because we have specified an instance UUID.
         token = util.get_api_token(
             'http://%s:%d' % (placed_on, config.parsed.get('API_PORT')),
-            namespace=namespace)
+            namespace='system')
         r = requests.request('POST',
                              'http://%s:%d/instances'
                              % (placed_on,
@@ -811,8 +828,18 @@ class Instances(Resource):
                     n.ensure_mesh()
                     n.update_dhcp()
 
-            with util.RecordedOperation('instance creation', instance) as _:
-                instance.create(lock=lock)
+            libvirt = util.get_libvirt()
+            try:
+                with util.RecordedOperation('instance creation',
+                                            instance) as _:
+                    instance.create(lock=lock)
+
+            except libvirt.libvirtError as e:
+                code = e.get_error_code()
+                if code in (libvirt.VIR_ERR_CONFIG_UNSUPPORTED,
+                            libvirt.VIR_ERR_XML_ERROR):
+                    return error_with_cleanup(400, e.get_error_message())
+                raise e
 
             for iface in db.get_instance_interfaces(instance.db_entry['uuid']):
                 db.update_network_interface_state(iface['uuid'], 'created')
@@ -823,7 +850,7 @@ class Instances(Resource):
     def delete(self, confirm=False, namespace=None):
         """Delete all instances in the namespace."""
 
-        if confirm != True:
+        if confirm is not True:
             return error(400, 'parameter confirm is not set true')
 
         if get_jwt_identity() == 'system':
@@ -978,7 +1005,7 @@ class Interface(Resource):
 
         n = net.from_db(ni['network_uuid'])
         if not n:
-            LOG.info('network(%s): network not found, genuinely missing'
+            LOG.info('network(%s): network not found or deleted'
                      % ni['network_uuid'])
             return error(404, 'interface network not found')
 
@@ -1007,7 +1034,7 @@ class InterfaceFloat(Resource):
 
         n = net.from_db(ni['network_uuid'])
         if not n:
-            LOG.info('network(%s): network not found, genuinely missing'
+            LOG.info('network(%s): network not found or deleted'
                      % ni['network_uuid'])
             return error(404, 'network not found')
 
@@ -1048,7 +1075,7 @@ class InterfaceDefloat(Resource):
 
         n = net.from_db(ni['network_uuid'])
         if not n:
-            LOG.info('network(%s): network not found, genuinely missing'
+            LOG.info('network(%s): network not found or deleted'
                      % ni['network_uuid'])
             return error(404, 'network not found')
 
@@ -1124,7 +1151,7 @@ class InstanceConsoleData(Resource):
     def get(self, instance_uuid=None, length=None, instance_from_db=None, instance_from_db_virt=None):
         try:
             length = int(length)
-        except:
+        except Exception:
             return error(400, 'length is not an integer')
 
         resp = flask.Response(
@@ -1140,13 +1167,16 @@ class Image(Resource):
     def post(self, url=None):
         db.add_event('image', url, 'api', 'cache', None, None)
 
-        with util.RecordedOperation('cache image', url) as _:
-            image_url = images.resolve(url)
-            hashed_image_path, info, image_dirty, resp = \
-                images.requires_fetch(image_url)
+        hashed = images.hash_image_url(url)
+        with db.get_lock('sf/images/%s' % hashed) as image_lock:
+            with util.RecordedOperation('cache image', url) as _:
+                image_url = images.resolve(url)
+                hashed_image_path, info, image_dirty, resp = \
+                    images.requires_fetch(image_url)
 
-            if image_dirty:
-                images.fetch(hashed_image_path, info, resp)
+                if image_dirty:
+                    images.fetch(hashed_image_path, info,
+                                 resp, locks=[image_lock])
 
 
 def _delete_network(network_from_db):
@@ -1236,7 +1266,7 @@ class Networks(Resource):
             if config.parsed.get('NODE_IP') == config.parsed.get('NETWORK_NODE_IP'):
                 n = net.from_db(network['uuid'])
                 if not n:
-                    LOG.info('network(%s): network not found, genuinely missing'
+                    LOG.info('network(%s): network not found or deleted'
                              % network['uuid'])
                     return error(404, 'network not found')
 
@@ -1270,7 +1300,7 @@ class Networks(Resource):
     def delete(self, confirm=False, namespace=None):
         """Delete all networks in the namespace."""
 
-        if confirm != True:
+        if confirm is not True:
             return error(400, 'parameter confirm is not set true')
 
         if get_jwt_identity() == 'system':
@@ -1285,20 +1315,20 @@ class Networks(Resource):
                 return error(401, 'you cannot delete other namespaces')
             namespace = get_jwt_identity()
 
-        networks_del =[]
-        for net in list(db.get_networks(all=all, namespace=namespace)):
-            if net['uuid'] == 'floating':
+        networks_del = []
+        for n in list(db.get_networks(all=all, namespace=namespace)):
+            if n['uuid'] == 'floating':
                 continue
 
             # If a network is in use, you should rethink the whole request
-            if len(list(db.get_network_interfaces(net['uuid']))) > 0:
+            if len(list(db.get_network_interfaces(n['uuid']))) > 0:
                 return error(403, 'cannot delete an in use network')
 
-            if net['state'] == 'deleted':
+            if n['state'] == 'deleted':
                 continue
 
-            _delete_network(net)
-            networks_del.append(net['uuid'])
+            _delete_network(n)
+            networks_del.append(n['uuid'])
 
         return networks_del
 
@@ -1313,6 +1343,16 @@ class NetworkEvents(Resource):
         return list(db.get_events('network', network_uuid))
 
 
+class NetworkInterfaces(Resource):
+    @jwt_required
+    @arg_is_network_uuid
+    @requires_network_ownership
+    def get(self, network_uuid=None, network_from_db=None):
+        db.add_event('network', network_uuid,
+                     'api', 'get interfaces', None, None)
+        return list(db.get_network_interfaces(network_uuid))
+
+
 class NetworkMetadatas(Resource):
     @jwt_required
     @arg_is_network_uuid
@@ -1377,7 +1417,7 @@ class DeployNetworkNode(Resource):
                      'network node', 'deploy', None, None)
         n = net.from_db(passed_uuid)
         if not n:
-            LOG.info('network(%s): network not found, genuinely missing'
+            LOG.info('network(%s): network not found or deleted'
                      % passed_uuid)
             return error(404, 'network not found')
 
@@ -1394,7 +1434,7 @@ class UpdateDHCP(Resource):
                      'network node', 'update dhcp', None, None)
         n = net.from_db(passed_uuid)
         if not n:
-            LOG.info('network(%s): network not found, genuinely missing'
+            LOG.info('network(%s): network not found or deleted'
                      % passed_uuid)
             return error(404, 'network not found')
 
@@ -1410,7 +1450,7 @@ class RemoveDHCP(Resource):
                      'network node', 'remove dhcp', None, None)
         n = net.from_db(passed_uuid)
         if not n:
-            LOG.info('network(%s): network not found, genuinely missing'
+            LOG.info('network(%s): network not found or deleted'
                      % passed_uuid)
             return error(404, 'network not found')
 
@@ -1455,6 +1495,7 @@ api.add_resource(Image, '/images')
 api.add_resource(Networks, '/networks')
 api.add_resource(Network, '/networks/<network_uuid>')
 api.add_resource(NetworkEvents, '/networks/<network_uuid>/events')
+api.add_resource(NetworkInterfaces, '/networks/<network_uuid>/interfaces')
 api.add_resource(NetworkMetadatas, '/networks/<network_uuid>/metadata')
 api.add_resource(NetworkMetadata,
                  '/networks/<network_uuid>/metadata/<key>')
@@ -1474,7 +1515,7 @@ class monitor(object):
         processutils.execute(
             ('gunicorn3 --workers 10 --bind 0.0.0.0:%d '
              '--log-syslog --log-syslog-prefix sf '
-             '--timeout 300 --name "sf api" '
+             '--timeout 600 --name "sf api" '
              'shakenfist.external_api.app:app'
              % config.parsed.get('API_PORT')),
             shell=True, env_variables=os.environ)
