@@ -1,7 +1,6 @@
 # Copyright 2019 Michael Still
 
 import base64
-import errno
 import jinja2
 import logging
 from logging import handlers as logging_handlers
@@ -17,7 +16,6 @@ from oslo_concurrency import processutils
 
 from shakenfist import config
 from shakenfist import db
-from shakenfist import db
 from shakenfist import images
 from shakenfist import net
 from shakenfist import util
@@ -29,9 +27,10 @@ LOG.addHandler(logging_handlers.SysLogHandler(address='/dev/log'))
 
 
 def from_definition(uuid=None, name=None, disks=None, memory_mb=None,
-                    vcpus=None, ssh_key=None, user_data=None, owner=None):
+                    vcpus=None, ssh_key=None, user_data=None, owner=None,
+                    video=None):
     db_entry = db.create_instance(uuid, name, vcpus, memory_mb, disks,
-                                  ssh_key, user_data, owner)
+                                  ssh_key, user_data, owner, video)
     return Instance(db_entry)
 
 
@@ -143,6 +142,8 @@ class Instance(object):
         self.db_entry['block_devices']['finalized'] = False
 
     def create(self, lock=None):
+        db.update_instance_state(self.db_entry['uuid'], 'creating')
+
         # Ensure we have state on disk
         if not os.path.exists(self.instance_path):
             LOG.debug('%s: Creating instance storage at %s' %
@@ -159,17 +160,19 @@ class Instance(object):
             modified_disks = []
             for disk in self.db_entry['block_devices']['devices']:
                 if disk.get('base'):
-                    with util.RecordedOperation('fetch image', self) as _:
-                        image_url = images.resolve(disk['base'])
-                        hashed_image_path, info, image_dirty, resp = \
-                            images.requires_fetch(image_url)
+                    hashed = images.hash_image_url(disk['base'])
+                    with db.get_lock('sf/images/%s' % hashed) as image_lock:
+                        with util.RecordedOperation('fetch image', self) as _:
+                            image_url = images.resolve(disk['base'])
+                            hashed_image_path, info, image_dirty, resp = \
+                                images.requires_fetch(image_url)
 
-                        if image_dirty:
-                            hashed_image_path = images.fetch(hashed_image_path,
-                                                             info, resp, lock=lock)
-                        else:
-                            hashed_image_path = '%s.v%03d' % (
-                                hashed_image_path, info['version'])
+                            if image_dirty:
+                                hashed_image_path = images.fetch(hashed_image_path,
+                                                                 info, resp, locks=[lock, image_lock])
+                            else:
+                                hashed_image_path = '%s.v%03d' % (
+                                    hashed_image_path, info['version'])
 
                     try:
                         cd = pycdlib.PyCdlib()
@@ -256,8 +259,9 @@ class Instance(object):
             except Exception:
                 pass
 
-        with util.RecordedOperation('release network addreses', self) as _:
+        with util.RecordedOperation('release network addresses', self) as _:
             for ni in db.get_instance_interfaces(self.db_entry['uuid']):
+                db.update_network_interface_state(ni['uuid'], 'deleted')
                 with db.get_lock('sf/ipmanager/%s' % ni['network_uuid'],
                                  ttl=120) as _:
                     ipm = db.get_ipmanager(ni['network_uuid'])
@@ -342,7 +346,8 @@ class Instance(object):
                     'id': devname,
                     'name': devname,
                     'mtu': 1450,
-                    'type': 'physical',
+                    'type': 'vif',
+                    'vif_id': iface['uuid']
                 }
             )
 
@@ -425,7 +430,9 @@ class Instance(object):
             networks=networks,
             instance_path=self.instance_path,
             console_port=self.db_entry['console_port'],
-            vdi_port=self.db_entry['vdi_port']
+            vdi_port=self.db_entry['vdi_port'],
+            video_model=self.db_entry['video']['model'],
+            video_memory=self.db_entry['video']['memory']
         )
 
         with open(self.xml_file, 'w') as f:
@@ -463,6 +470,8 @@ class Instance(object):
         db.update_instance_power_state(
             self.db_entry['uuid'],
             util.extract_power_state(libvirt, instance))
+        db.add_event(
+            'instance', self.db_entry['uuid'], 'poweron', 'complete', None, None)
 
     def power_off(self):
         libvirt = util.get_libvirt()
@@ -482,6 +491,9 @@ class Instance(object):
         except libvirt.libvirtError as e:
             LOG.error('%s: Failed to delete domain: %s' % (self, e))
 
+        db.add_event(
+            'instance', self.db_entry['uuid'], 'poweron', 'complete', None, None)
+
     def _snapshot_device(self, source, destination):
         images.snapshot(source, destination)
 
@@ -525,6 +537,9 @@ class Instance(object):
         else:
             instance.reset()
 
+        db.add_event(
+            'instance', self.db_entry['uuid'], 'reboot', 'complete', None, None)
+
     def pause(self):
         libvirt = util.get_libvirt()
         instance = self._get_domain()
@@ -533,6 +548,9 @@ class Instance(object):
             self.db_entry['uuid'],
             util.extract_power_state(libvirt, instance))
 
+        db.add_event(
+            'instance', self.db_entry['uuid'], 'pause', 'complete', None, None)
+
     def unpause(self):
         libvirt = util.get_libvirt()
         instance = self._get_domain()
@@ -541,6 +559,9 @@ class Instance(object):
             self.db_entry['uuid'],
             util.extract_power_state(libvirt, instance))
 
+        db.add_event(
+            'instance', self.db_entry['uuid'], 'unpause', 'complete', None, None)
+
     def get_console_data(self, length):
         console_path = os.path.join(self.instance_path, 'console.log')
         if not os.path.exists(console_path):
